/** written by olaf booij
 *  public domain */

#ifndef QRSMURRIE_IO_HPP_INCLUDE
#define QRSMURRIE_IO_HPP_INCLUDE

#include <boost/filesystem.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/shared_ptr.hpp>
#include <string>
#include <fstream>
#include <cmath>

namespace qrsmurrie {

  template< typename QRcodeT > // = QRcode&
  void printQRcode(const QRcodeT& qrcode)
  {
    unsigned fenoI = 0;
    for(unsigned fenoRowI = 0; fenoRowI < qrcode.width; ++fenoRowI)
    {
      for(unsigned fenoColI = 0; fenoColI < qrcode.width; ++fenoColI)
      {
        if(getQRValue(qrcode, fenoI++))
          std::cout << "O";
        else
          std::cout << " ";
      }
      std::cout << std::endl;
    }
  }
  template< typename MonoImageT > // = QRcode&
  void printImage(const MonoImageT& image)
  {
    const unsigned imageSize = std::sqrt(image.size());
    unsigned imageI = 0;
    for(unsigned imageRowI = 0; imageRowI < imageSize; ++imageRowI)
    {
      for(unsigned imageColI = 0; imageColI < imageSize; ++imageColI)
      {
        if(image.at(imageI++))
          std::cout << "O";
        else
          std::cout << " ";
      }
      std::cout << std::endl;
    }
  }

  template< typename QRcodeT > // = QRcode&
  void saveQRcodeAsPGM(const QRcodeT& qrcode, const std::string filename)
  {
    typedef boost::filesystem::ofstream OutFile;
    typedef boost::shared_ptr< OutFile > OutFilePtr; // should be using auto...
    OutFilePtr imageFilePtr(new OutFile(filename));
    if(! imageFilePtr.get()->is_open())
      std::cout << "openFile: file " << filename << " can not be opened (or so it seems)" << std::endl;
    {
      OutFile& imageFile = *imageFilePtr;
      imageFile << "P2" << std::endl;
      const unsigned imageSize = qrcode.width;
      imageFile << imageSize << " " << imageSize << std::endl;
      imageFile << 255 << std::endl;
      unsigned fenoI = 0;
      for(unsigned fenoRowI = 0; fenoRowI < imageSize; ++fenoRowI)
      {
        for(unsigned fenoColI = 0; fenoColI < imageSize; ++fenoColI)
        {
          if(getQRValue(qrcode, fenoI++))
            imageFile << "0 ";
          else
            imageFile << "255 "; // CHECKME, or the other way round? 
        }
        imageFile << std::endl;
      }
    }
  }

} // namespace qrsmurrie

#endif // QRSMURRIE_IO_HPP_INCLUDE

