/** written by olaf booij
 *  public domain */

#include <iostream>
#include <string>
#include <cstdlib>
#include <ctime>
#include <qrsmurrie/qrencode_wrapper.hpp>
#include <qrsmurrie/random_string.hpp>
#include <qrsmurrie/image_compare.hpp>
#include <qrsmurrie/io.hpp>
#include <qrsmurrie/permSort.hpp>
#include <qrsmurrie/flok_of_strings.hpp>
#include <qrsmurrie/mutate.hpp>
#include <qrsmurrie/crossover.hpp>
#include <qrsmurrie/sumqrs.hpp>
//#include <boost/timer/timer.hpp>
#include <boost/progress.hpp>

using namespace qrsmurrie;
int main()
{
  using namespace std;
  srand(time(NULL));
  const unsigned stringSize = 16;
  MonoImage image = generateBlackImage(getNrOfQRValuesGivenStringSize(stringSize));
  printImage(image);
  cout << "image size = " << image.size() << std::endl;

  string best_string;
  const unsigned nr_of_strings = 100;
  Flok flok = randomFlok(nr_of_strings, stringSize);
  const unsigned nr_of_iterations = 1000;
  boost::progress_display progressBar(nr_of_iterations);
  for(unsigned iter = 0; iter < nr_of_iterations; ++iter)
  {
    std::vector< QRcodePtr > qrcodes;
    foreach(const std::string text, flok)
      qrcodes.push_back(encodeQR(text));
    QRsum qrsum = sumQRs(qrcodes);
    std::vector< double > weights;
    foreach(const unsigned qrsumValue, qrsum)
      weights.push_back((double) qrsumValue / qrcodes.size());
    std::vector< double > fitness;
    foreach(const QRcodePtr qrcode, qrcodes)
      fitness.push_back(weightedSimilarityImageQR(image, *qrcode, weights));
    std::vector< std::size_t > fitnessPerm = permSort(fitness);
    flok = applyPerm2(fitnessPerm, flok);
    //fitness = applyPerm2(fitnessPerm, fitness);
    //foreach(double fitnes, fitness)
    //  cout << fitnes << endl;
    //qrcodes = applyPerm2(fitnessPerm, qrcodes);
    if(iter % 10 == 0)
    {
      std::vector< unsigned > weaknesses;
      foreach(const QRcodePtr qrcode, qrcodes)
        weaknesses.push_back(differenceImageQR(image, *qrcode));
      cout << *std::min_element(weaknesses.begin(), weaknesses.end()) << endl;
    }
    // mutating and crossovering
    //// mostly crossover
    //const int part = 4;
    //for(unsigned replaceI = 0; replaceI < round(flok.size()/part); ++replaceI)
    //{
    //  unsigned mother = flok.size() - flok.size() / part + std::rand() % (flok.size() / part);
    //  unsigned father = flok.size() - flok.size() / part + std::rand() % (flok.size() / part);
    //  flok.at(replaceI) = crossover(flok.at(mother), flok.at(father));
    //}
    //foreach(std::string& text, flok)
    //  text = mutate(text, .3);
    // pure mutation
    const int part = 4;
    for(unsigned replaceI = 0; replaceI < round(flok.size()/part); ++replaceI)
    {
      if(std::rand() % 2 == 1)
      {
        unsigned parent = flok.size() - flok.size() / part + std::rand() % (flok.size() / part);
        flok.at(replaceI) = mutate(flok.at(parent));
      }
      else
      {
        unsigned mother = flok.size() - flok.size() / part + std::rand() % (flok.size() / part);
        unsigned father = flok.size() - flok.size() / part + std::rand() % (flok.size() / part);
        flok.at(replaceI) = crossover(flok.at(mother), flok.at(father));
      }
    }
    ++progressBar;
  }
  {
    std::vector< unsigned > weaknesses;
    foreach(const std::string text, flok)
      weaknesses.push_back(differenceImageQR(image, *encodeQR(text)));
    unsigned bestI = std::min_element(weaknesses.begin(), weaknesses.end()) - weaknesses.begin(); 
    cout << weaknesses.at(bestI) << endl;
    printQRcode(*encodeQR(flok.at(bestI)));
    std::cout << ">" << flok.at(bestI) << "<" << std::endl;
    std::string testFileName("test.pgm");
    saveQRcodeAsPGM(*encodeQR(flok.at(bestI)), testFileName);
  }
  return(0);
}
